* Serial bootstrap loader for Tiny030
* 2/24/19
******************************************
* T030Load Tiny030 bootstrap loader
* (C)2019 Huichien Shen
******************************************
SerData  equ $FFFFF000                 * serial port transmit/receive data
SerStat  equ $FFFFF001                 * serial port status
         org 0
         dc.l $8000                 * stack
         dc.l start                  * reset start address

start: 
         move.l #$4000,sp
;         move.b #'>',SerData        * sign on  
         lea signon,a0
chkTE:
         btst.b #0,SerStat
         beq.s chkTE
         move.b (a0)+,SerData
         bne.s chkTE         
         clr.l d5 
cmd:
         bsr.s cinq                   *get character input
         cmp.b #$d,d0                  * carriage return?
         bne.s doload2
         tst.b d5
         beq.s doload3
         move.b #'?',SerData        *error, put out '?'
         bra.s cmd
doload3:
         move.b #'.',SerData        *no error, put out '.'
         bra.s cmd                  
doload2:         
         cmp.b #'S',d0                 *S record?
         bne.s cmd                *get S record
doSload:
         bsr.s cinq                 * get 2nd character
         cmp.b #'1',d0              *S1 record
         beq.s doS1
         cmp.b #'0',d0              *S0 record?
         beq.s cmd                    *discard S0 record
         tst.b d5                   *do not jump to $400 if there are errors
         bne.s start
         bra $400                   *jump to $400 for everything else

doS1:
* 2-byte address field
	clr.l d4
	moveq #1,d2		* get Srecord byte count
	bsr.s char2hex
	add.b d1,d4		* accumulate checksum
	move.w d1,d3
	subq #4,d3		* don't count CRC byte nor 2-byte address		
	moveq #3,d2		* get 2-byte starting address
	bsr.s char2hex
	add.b d1,d4		* accumulate checksum, lower address
	ror.w #8,d1		
	add.b d1,d4		* accumulate checksum, higher address
	ror.w #8,d1		* restore
	move.l d1,a0
srdata:
	moveq #1,d2		* get byte data
	bsr.s char2hex
	add.b d1,d4		* accumulate checksum
	move.b d1,(a0)		* store value to destination
	cmp.b (a0)+,d1		* write verify
	beq.s srdatax
	moveq #1,d5
srdatax:	
	dbra d3,srdata
	not.b d4		         * checksum expected
	moveq #1,d2
	bsr.s char2hex
	cmp.b d1,d4
	beq cmd
	moveq #1,d5                * send out a question mark
	bra cmd
cinq:
         btst.b #1,SerStat            *look for RxRdy set
         beq.s cinq
         move.b SerData,d0          *get the input character
;         move.b d0,SerData          * echo
         rts        
*
* subroutine to get prescribed number of characters in d2+1, convert to hex and put in
* d1 
char2hex:
	clr.l d1		         * important to clear all d1 when start
c2h1:
         bsr.s cinq                   *get input character in d0
	bsr.s asc2hex
	cmp.b #$FF,d0
	bne.s c2h1_1
	moveq #1,d5                * mark this as an error and go on
c2h1_1:
	lsl.l #4,d1		
	add.b d0,d1		* append to existing value
	dbra d2,c2h1
	rts
*
* check for valid ascii representation of hexdecimal
* d0 contains input, return 0xFF if invalid, otherwise hex value
asc2hex:
	cmp.b #'f',d0		
	bhi.s aschex9		* not valid, >'f'
	cmp.b #'a'-1,d0
	bhi.s aschex1		* 'f'> valid >'a'
	cmp.b #'F',d0		
	bhi.s aschex9		* not valid, >'F'
	cmp.b #'A'-1,d0
	bhi.s aschex3		* 'F'> valid >'A'
	cmp.b #'9',d0
	bhi.s aschex9		* not valid >'9'
	cmp.b #'0'-1,d0
	bhi.s aschex2		* '9'> valid >'0'
aschex9:
	moveq #-1,d0		* return 0xFF in not hex
	bra.s aschex0
aschex3:
	sub.b #$37,d0
	bra.s aschex0
aschex2:
	sub.b #$30,d0
	bra.s aschex0
aschex1:
	sub.b #$57,d0		* 'a'-'f'
aschex0:	
	rts
signon:  dc.b $a,$d,'V0.2 >',0            
      
         end 0
         







*~Font name~Courier New~
*~Font size~10~
*~Tab type~1~
*~Tab size~9~
